unit rzip_interface;
{<
  rzip_interface contains universal interface between rXMLReport and
  rzip.dll or Abbrevia package according compiler swich USE_ZIPDLL.

  @author(Rosinsky Software - <http://www.rosinsky.cz/delphi>)
  @created(May 2016)
  @lastmod(May 2016)
}

interface

{$INCLUDE _rosi.inc}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  { rzip_interface contains universal interface between rXMLReport and
  rzip.dll or Abbrevia package according compiler swich USE_ZIPDLL }
  Trzip_interface = record
    Loaded: boolean;
    procedure Load;
    procedure UnLoad;
    function OpenArchive(ZipFile: string): integer;
    function CloseArchive(Save: boolean): integer;
    function GetFileList(var ListText: string): integer;
    function ExtractAllFiles(OutputPath: string): integer;
    function ExtractFile(ExtractFile, OutputFile: string): integer;
    function ExtractToBuffer(ExtractFile: string; Buffer: Pointer; var Length: Int64): integer;
    //function ExtractContent(ExtractFile: string; var Content: PChar): integer;
    function ExtractToString(ExtractFile: string; var Content: string): integer;
    function AddFile(NewFile, InputFile: string): integer;
    function AddFromBuffer(NewFile: string; Buffer: Pointer; Length: Int64): integer;
    //function AddContent(NewFile: string; Content: PChar): integer;
    function AddFromString(NewFile: string; Content: string): integer;
    function GetLastErrorMsg: string;
  end;

var
  rzip: Trzip_interface;

implementation

uses
{$IFNDEF NOTUSE_ZIPDLL}
{$IFDEF USE_ZIPDLL}
{$DEFINE RZIPDLL}
{$ENDIF}
{$ENDIF}
{$IFNDEF RZIPDLL} rzipAbbrevia_interface, {$ENDIF}
  rtool, rdlg, rstring;
{$IFDEF RZIPDLL}

const
  DLLFileName = 'rzip.dll';

type
  rzip_TOpenArchive = function(ZipFile: PChar): integer; stdcall;
  rzip_TCloseArchive = function(Save: boolean): integer; stdcall;
  rzip_TGetFileList = function(var ListText: PChar): integer; stdcall;

  rzip_TExtractAllFiles = function(OutputPath: PChar): integer; stdcall;
  rzip_TExtractFile = function(ExtractFile, OutputFile: PChar): integer; stdcall;
  rzip_TExtractToBuffer = function(ExtractFile: PChar; Buffer: Pointer; var Length: Int64): integer; stdcall;
  rzip_TExtractContent = function(ExtractFile: PChar; var Content: PChar): integer; stdcall;

  rzip_TAddFile = function(NewFile, InputFile: PChar): integer; stdcall;
  rzip_AddFromBuffer = function(NewFile: PChar; Buffer: Pointer; Length: Int64): integer; stdcall;
  rzip_TAddContent = function(NewFile, Content: PChar): integer; stdcall;
  rzip_GetLastErrorMsg = function: PChar; stdcall;

  TZipDLL_interface = record
    Loaded: boolean;
    DLLHandle: THandle;
    OpenArchive: rzip_TOpenArchive;
    CloseArchive: rzip_TCloseArchive;
    GetFileList: rzip_TGetFileList;
    ExtractAllFiles: rzip_TExtractAllFiles;
    ExtractFile: rzip_TExtractFile;
    ExtractToBuffer: rzip_TExtractToBuffer;
    ExtractContent: rzip_TExtractContent;
    AddFile: rzip_TAddFile;
    AddFromBuffer: rzip_AddFromBuffer;
    AddContent: rzip_TAddContent;
    GetLastErrorMsg: rzip_GetLastErrorMsg;
    procedure LoadDll;
    procedure UnloadDll;
  end;

var
  zipdll: TZipDLL_interface;

procedure TZipDLL_interface.LoadDll;
begin
  zipdll.Loaded:=false;
  DLLHandle:=LoadLibrary(DLLFileName);
  if DLLHandle <= 0 then
    DlgE(Format(sDLLNotFound, [DLLFileName]))
  else
  begin
    @zipdll.OpenArchive:=GetProcAddress(DLLHandle, 'OpenArchive');
    @zipdll.CloseArchive:=GetProcAddress(DLLHandle, 'CloseArchive');
    @zipdll.GetFileList:=GetProcAddress(DLLHandle, 'GetFileList');
    @zipdll.ExtractAllFiles:=GetProcAddress(DLLHandle, 'ExtractAllFiles');
    @zipdll.ExtractFile:=GetProcAddress(DLLHandle, 'ExtractFile');
    @zipdll.ExtractToBuffer:=GetProcAddress(DLLHandle, 'ExtractToBuffer');
    @zipdll.ExtractContent:=GetProcAddress(DLLHandle, 'ExtractContent');
    @zipdll.AddFile:=GetProcAddress(DLLHandle, 'AddFile');
    @zipdll.AddFromBuffer:=GetProcAddress(DLLHandle, 'AddFromBuffer');
    @zipdll.AddContent:=GetProcAddress(DLLHandle, 'AddContent');
    @zipdll.GetLastErrorMsg:=GetProcAddress(DLLHandle, 'GetLastErrorMsg');
    zipdll.Loaded:=true;
  end;
end;

procedure TZipDLL_interface.UnloadDll;
begin
  FreeLibrary(DLLHandle);
  DLLHandle:=0;
  zipdll.Loaded:=false;
end;

procedure Trzip_interface.Load;
begin
  zipdll.LoadDll;
  Loaded:=zipdll.Loaded;
end;

procedure Trzip_interface.UnLoad;
begin
  zipdll.UnloadDll;
end;

function Trzip_interface.OpenArchive(ZipFile: string): integer;
begin
  Result:=zipdll.OpenArchive(PChar(ZipFile));
end;

function Trzip_interface.CloseArchive(Save: boolean): integer;
begin
  Result:=zipdll.CloseArchive(Save);
end;

function Trzip_interface.GetFileList(var ListText: string): integer;
var
  P: PChar;
begin
  P:='';
  Result:=zipdll.GetFileList(P);
  ListText:=P;
end;

function Trzip_interface.ExtractAllFiles(OutputPath: string): integer;
begin
  Result:=zipdll.ExtractAllFiles(PChar(OutputPath));
end;

function Trzip_interface.ExtractFile(ExtractFile, OutputFile: string): integer;
begin
  Result:=zipdll.ExtractFile(PChar(ExtractFile), PChar(OutputFile));
end;

function Trzip_interface.ExtractToBuffer(ExtractFile: string; Buffer: Pointer; var Length: Int64): integer;
begin
  Result:=zipdll.ExtractToBuffer(PChar(ExtractFile), Buffer, Length);
end;

{
function Trzip_interface.ExtractContent(ExtractFile: string; var Content: PChar): integer;
begin
  Result:=zipdll.ExtractContent(PChar(ExtractFile), Content);
end;
}

function Trzip_interface.ExtractToString(ExtractFile: string; var Content: string): integer;
var
  P: PChar;
begin
  P:='';
  Result:=zipdll.ExtractContent(PChar(ExtractFile), P);
  Content:=P;
end;

function Trzip_interface.AddFile(NewFile, InputFile: string): integer;
begin
  Result:=zipdll.AddFile(PChar(NewFile), PChar(InputFile));
end;

function Trzip_interface.AddFromBuffer(NewFile: string; Buffer: Pointer; Length: Int64): integer;
begin
  Result:=zipdll.AddFromBuffer(PChar(NewFile), Buffer, Length);
end;

{
function Trzip_interface.AddContent(NewFile: string; Content: PChar): integer;
begin
  Result:=zipdll.AddContent(PChar(NewFile), Content);
end;
}

function Trzip_interface.AddFromString(NewFile: string; Content: string): integer;
begin
  Result:=zipdll.AddContent(PChar(NewFile), PChar(Content));
end;

function Trzip_interface.GetLastErrorMsg: string;
begin
  Result:=zipdll.GetLastErrorMsg;
end;
{$ELSE}

procedure Trzip_interface.Load;
begin
  rAbZipObject_Create;
  Loaded:=true;
end;

procedure Trzip_interface.UnLoad;
begin
  eAbZipObject_Free;
end;

function Trzip_interface.OpenArchive(ZipFile: string): integer;
begin
  Result:=rAbZipObject.OpenArchive(ZipFile);
end;

function Trzip_interface.CloseArchive(Save: boolean): integer;
begin
  Result:=rAbZipObject.CloseArchive(Save);
end;

function Trzip_interface.GetFileList(var ListText: string): integer;
begin
  Result:=rAbZipObject.GetFileList(ListText);
end;

function Trzip_interface.ExtractAllFiles(OutputPath: string): integer;
begin
  Result:=rAbZipObject.ExtractAllFiles(OutputPath);
end;

function Trzip_interface.ExtractFile(ExtractFile, OutputFile: string): integer;
begin
  Result:=rAbZipObject.ExtractFile(ExtractFile, OutputFile);
end;

function Trzip_interface.ExtractToBuffer(ExtractFile: string; Buffer: Pointer; var Length: Int64): integer;
begin
  Result:=rAbZipObject.ExtractToBuffer(ExtractFile, Buffer, Length);
end;

{
function Trzip_interface.ExtractContent(ExtractFile: string; var Content: PChar): integer;
begin
  Result:=rAbZipObject.ExtractContent(ExtractFile, Content);
end;
}

function Trzip_interface.ExtractToString(ExtractFile: string; var Content: string): integer;
begin
  Result:=rAbZipObject.ExtractToString(ExtractFile, Content);
end;

function Trzip_interface.AddFile(NewFile, InputFile: string): integer;
begin
  Result:=rAbZipObject.AddFile(NewFile, InputFile);
end;

function Trzip_interface.AddFromBuffer(NewFile: string; Buffer: Pointer; Length: Int64): integer;
begin
  Result:=rAbZipObject.AddFromBuffer(NewFile, Buffer, Length);
end;

{
function Trzip_interface.AddContent(NewFile: string; Content: PChar): integer;
begin
  Result:=rAbZipObject.AddContent(NewFile, Content);
end;
}

function Trzip_interface.AddFromString(NewFile: string; Content: string): integer;
begin
  Result:=rAbZipObject.AddFromString(NewFile, Content);
end;

function Trzip_interface.GetLastErrorMsg: string;
begin
  Result:=rAbZipObject.GetLastErrorMsg;
end;
{$ENDIF}

initialization

rzip.Loaded:=false;

end.
